UNIT MFCHT;
{ c Petr Mikulk - Brno - 1991 }
{$F+,O+,S-,X+,D-,L-,R-}

INTERFACE

uses crt, DefFloat, CompNum, Dim3, PmElFun;

type String10 = string[10];
     String8  = string[8];


{ ---- physical constants ----- }

const
  c_light   = 2.997924E8;    { m/s }
  h_Planck  = 6.626176E-34;  { Js }
  hbar_Planck = 1.0545887E-34; { Js }
  e_el      = 1.6021892E-19; { C }
  m_el      = 9.109534E-31;  { kg }
  m_p       = 1.6726485E-27; { kg }
  m_n       = 1.6749543E-27; { kg }
  m_unit    = 1.6605655E-27; { kg }
  mi_Bohr   = 9.274078E-24;  { Am^2 nebo J/T }
  a0_Bohr   = 5.2917706E-11; { m }
  r_el      = 2.8179380E-15; { m = e^2 / (4 pi e0 m_el c^2) }
  N_Avog    = 6.022045e23;   { 1/mol }
  k_Boltz   = 1.380662E-23;  { J/K }
  R_plyn    = 8.31441;       { J / mol / K }
  Kappa     = 6.6720E-11;    { N m^2 kg^-2 }
  g_grav    = 9.80665;       { m s^-2 }
  epsilon0  = 8.854187818E-12; { F / m }
  jedna_4pie0 = 1/4/pi/epsilon0;
  mi0       = 4E-7 * pi;     { H / m }


{ --- ordering of atoms --- }

const atomH  = 0;  atomSi = 1; atomGe = 2; atomAl = 3;
      atomGa = 4;  atomAs = 5;
      atomFe = 6;  atomY  = 7; atomGd = 8;
      atomTb = 9;  atomW  = 10; atomO  = 11;
      atomNb = 12; atomSm = 13;
      atomLast = 13;

type TTypAtomu = atomH..atomLast;
     Array9F   = array [ 1..9 ] of float;
     Array10F  = array [ 1..10 ] of float;


{ ---- periodic system ---- }

const zetB  = 5;   zetC  = 6;   zetN  = 7;
      zetAl = 13;  zetSi = 14;  zetP  = 15;
      zetGa = 31;  zetGe = 32;  zetAs = 33;

      zetO  = 8;   zetFe = 26;  zetY  = 39;
      zetNb = 41;
      zetSm = 62;  zetGd = 64;  zetTb = 65;
      zetW  = 74;


{ ---- lattice parameters ---- }

const
  { diamant structure }

      lpGaAs = 5.65325e-10;
      lpAlAs = 5.6620e-10;
        lpAlAsonGaAs = 5.6664e-10; { pri rustu na GaAs }
      lpSi   = 5.4309e-10;
      lpGe   = 5.6575e-10;
      lpC    = 3.567e-10; { diamant }
      lpGa5Al5As = ( lpGaAs + lpAlAs ) /2;


  { cubic - bcc }

      lpFe = 2.8664e-10;
      lpW  = 3.1648e-10;
      lpNb = 3.3066e-10;


  { hexagonal }
      lpY_a  = 3.64713e-10;  lpY_c  = 5.72856e-10;
      lpGd_a = 3.629e-10;    lpGd_c = 5.760e-10;
      lpTb_a = 3.592e-10;    lpTb_c = 5.675e-10;
      lpSapphire_a = 4.759e-10; lpSapphire_c = 12.991e-10;
      lpSm_a = 3.6291e-10;   lpSm_c = 26.203e-10;


{ ---- relative masses ---- }

const mrB  = 10.82;     mrC  = 12.011;   mrN  = 14.0067;
      mrAl = 26.98154;  mrSi = 28.0855;  mrP  = 30.9737;
      mrGa = 69.72;     mrGe = 72.59;    mrAs = 74.9216;

      mrFe = 55.85;     mrY  = 88.91;    mrGd = 157.25;
      mrNb = 92.906;    mrSm = 150.36;
      mrTb = 158.93;    mrW  = 183.85;


{ ---- wavelengths ---- }

const
      CrKalfa1 = 2.289620e-10;
      FeKalfa1 = 1.93597e-10;
      CuKalfa1 = 1.540562E-10;   CuKalfa2 = 1.544390E-10;   CuKbeta1 = 1.392218E-10;
        CuKalfa = ( 2 * CuKalfa1 + CuKalfa2 ) / 3;   { 1.541838e-10 }
      MoKalfa1 = 0.709300E-10;   MoKalfa2 = 0.713590E-10;   MoKbeta1 = 0.632288E-10;
        MoKalfa = ( MoKalfa1 + MoKalfa2 ) /2;
      AgKalfa1 = 0.559408E-10;   AgKalfa2 = 0.563798E-10;   AgKbeta2 = 0.497069E-10;
        AgKalfa = ( AgKalfa1 + AgKalfa2 ) /2;

      AuKalfa1 = 0.180195e-10;  WKalfa1  = 0.209010e-10;
      TaKalfa1 = 0.215947e-10;  CoKalfa1 = 1.788965e-10;
      TiKalfa1 = 2.748510e-10;

const HowManyLambda = 11;
      WhichLambdaStr: array [1..HowManyLambda] of String8 =
        ('CUKALFA','CUKALFA1','CUKALFA2','MOKALFA','MOKALFA1','MOKALFA2',
         'AGKALFA','AGKALFA1','AGKALFA2','CRKALFA1','FEKALFA1');
      WhichLambda: array [1..HowManyLambda] of float =
        (CuKalfa,CuKalfa1,CuKalfa2,MoKalfa,MoKalfa1,MoKalfa2,
         AgKalfa,AgKalfa1,AgKalfa2,CrKalfa1,FeKalfa1);


{ --- Debye-Waller factors B ---}

      dwbGaAs = 0.596177e-20; { 283 K = the Debye temperature }
      dwbAlAs = 0.353084e-20; { 446 }
      dwbGe   = 0.558810e-20; { 292 }
      dwbSi   = 0.444083e-20; { 543 }
      dwbGa5Al5As = ( dwbGaAs + dwbAlAs ) /2;
      dwbFe   = 0.291911e-20; { 470 }
      dwbY    = 0.495128e-20; { 280 }
      dwbGd   = 0.542138e-20; { 200 }
      dwbTb   = 0;            { ? I don't know the Debye temperature }
      dwbW    = 0.120269e-20; { 400 }
      dwbSapphire = 0;        { ? }
      dwbNb   = 0;            { ? }
      dwbSm   = 0;            { ? }

{ --- procedures for calc. of atomic scattering factors --- }

procedure ASFSi ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFGe ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFAl ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFGa ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFAs ( STL: float; Lambda: float; var ASF: Complex );

procedure ASFO  ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFFe ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFY  ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFNb ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFSm ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFGd ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFTb ( STL: float; Lambda: float; var ASF: Complex );
procedure ASFW  ( STL: float; Lambda: float; var ASF: Complex );


const AtomScatFact: array [ atomSi..atomSm ] of Pointer =
        ( @ASFSi, @ASFGe, @ASFAl, @ASFGa, @ASFAs,
          @ASFFe, @ASFY,  @ASFGd, @ASFTb, @ASFW, @ASFO, @ASFNb,
          @ASFSm );

const KolikZnaRtg = 14;     { lib. usporadani }
      CoZnaRtg: array [1..KolikZnaRtg] of String10 =
        ( 'Si','Ge','GaAs','AlAs','AlAsonGaAs','Ga.5Al.5As',
          'Fe','Y','Gd','Tb','W','Sapphire', 'Nb', 'Sm' );

function RtgZnaLatku ( Latka: String10 ): boolean;

const HelpLambda = 'CuKalfa 1 2   MoKalfa 1 2   AgKalfa 1 2   CrKalfa1  FeKalfa1  error->CuKalfa';

function StringToLambda ( Lambda: string ): real;


IMPLEMENTATION


(*******  Atomic scattering factors for x-rays ****************
By: International  Tables for X-ray Crystallography,
The International Union of Crystallography, Birmingham, England,
1974.  Ozn. STL = sin(theta)/lambda. Pak:

              4                    2
f ( STL ) = suma a * exp ( -b * STL )  + c
            i=1   i          i

Coefficients for Analytical Approximation to the Scattering
Factors - Table 2.2B *****************************************)

procedure ASF1974 ( STL: float; Coef: Array9F; var ASF: Complex );
{ Stl je  sin theta / lambda }
var X: float; I: integer;
begin
X:=Coef[9];
STL:=-sqr(STL*1E-10);
for I:=0 to 3 do
    X:=X + Coef[2*I+1] * exp( STL * Coef[2*I+2] );
if STL=0 then X:=round(X); { vyrovnani chyby pro theta=0 }
Cnum(X,0,ASF);
end;


(********************** Table 2.3.1 ************************
   Real and Imaginary Dispersion Corrections for Atomic
Scattering Factors:  sf = asf1974 + sfr + i sfi

{ Int. Tables 1974:
      CrKalfa      FeKalfa       CuKalfa      MoKalfa      AgKalfa
Si   .355  .693   .311  .509    .244  .330   .072  .071   .042  .043
Al   .318  .522   .269  .381    .204  .246   .056  .052   .032  .031
Ga  -.570 1.569  -.841 1.168  -1.354  .777   .163 1.609   .249 1.059
Ge  -.462 1.786  -.717 1.331  -1.163  .886   .081 1.801   .228 1.190
As  -.365 2.022  -.607 1.508  -1.011 1.006  -.030 2.007   .196 1.332
}


{ Int. Tables 1992: }

        CrKalfa         FeKalfa         CuKalfa        MoKalfa         AgKalfa
Si   .3647  .6921    .3209  .5081    .2541   .3302    .0817 .0704    .0522  .0431
Al   .3260  .5212   .2774  .3807    .2130    .2455    .0645 .0514    .0313  .0522
Ga  -.4989 1.5674   -.7701 1.1663  -1.2846   .7763    .2307 1.6083   .3179 1.0589
Ge  -.3858 1.7841   -.6412 1.3291  -1.0885   .8855    .1547 1.8001   .3016 1.1903
As  -.2871 2.0194   -.5260 1.5069   -.9300  1.0051   -.0499 2.0058   .2758 1.3314
Fe -1.2935  .7620  -2.0554  .5649  -1.1336  3.1974    .3463  .8444   .5448  .3050
Y    .1263 3.9964    .0138 3.0054   -.2670  2.0244  -2.7962 3.5667  -.2879 2.4099
Gd -5.3778 5.7756  -7.1681 4.4280  -8.8380 11.9157   -.1653 3.9035  -.3741 2.5954
Tb -5.0951 6.1667  -6.5583 4.7292  -9.1472  9.1891   -.1723 4.1537  -.3496 2.7654
Sm -6.0570 5.0744  -9.3497 3.8839  -5.3236 12.2178   -.1638 3.4418  -.4287 2.2815
W  -3.9270 10.3696 -4.2486 8.0005  -5.4734  5.5774   -.8490 6.8722  -.4201 4.6430
O   0.0928 0.0731   0.0716 0.0518   0.0492 0.0322    0.0106 0.0060  0.0056 0.0036
Nb  0.1211 4.8761   0.1072 3.6768  -0.1121 2.4826   -2.0727 0.6215  -.8282 2.8404

***********************************************************)

procedure AddDispCorr ( B: Array10F; Lambda: float; var DispCorr: Complex );
var DC: Complex; Lambda09, Lambda11: float;
begin
Lambda09:=Lambda/0.9; Lambda11:=Lambda/1.1;
if (Lambda09>AgKalfa) and (Lambda11<AgKalfa) then Cnum(B[5],B[6],DC)
  else if (Lambda09>CrKalfa1) and (Lambda11<CrKalfa1) then Cnum(B[1],B[2],DC)
  else if (Lambda09>FeKalfa1) and (Lambda11<FeKalfa1) then Cnum(B[3],B[4],DC)
  else if (Lambda09>CuKalfa) and (Lambda11<CuKalfa) then Cnum(B[5],B[6],DC)
  else if (Lambda09>MoKalfa) and (Lambda11<MoKalfa) then Cnum(B[7],B[8],DC)
  else if (Lambda09>AgKalfa) and (Lambda11<AgKalfa) then Cnum(B[9],B[10],DC)
  else begin Cnum( 0, 0, DC );
       writeln('Dispersion correction for the wavelength ',Lambda,' is not in table - treated as 0.');
       end;
Cadd(DispCorr,DC,DispCorr);
end;

(***                 a1        b1        a2        b2        a3       b3       a4       b4       c    ***)

procedure ASFSi ( STL: float; Lambda: float; var ASF: Complex );
{ Si/v: maximum error 0.009. Pro Stl=0 je SF=13.9976 }
const A: Array9F = ( 6.29150,  2.43860,  3.03530,  32.3337,  1.98910, 0.678500,1.54100, 81.6937, 1.14070  );
(**
 { Si/val: maximum error 0.001. 0: 13.99906 }
 const A: Array9F = ( 5.66269,  2.66520,  3.07164,  38.6634,  2.62446, 0.916946,1.39320, 93.5458, 1.24707  );
 { Si/+4: maximum error 0.000. 0: 9.999987 }
 const A: Array9F = ( 4.43918,  1.64167,  3.20345,  3.43757,  1.19453, 0.214900,0.416530,6.65365, 0.746297 );
**)
{B:Array10F=(0.355, 0.693, 0.311, 0.509, 0.244, 0.330, 0.072, 0.071, 0.042, 0.043);   =1974}
 B:Array10F=(0.3647,0.6921,0.3209,0.5081,0.2541,0.3302,0.0817,0.0704,0.0522,0.0431); {=1992}
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;


procedure ASFGe ( STL: float; Lambda: float; var ASF: Complex );
(**
{ Ge/+4: maximum error 0.000. 0: 27.999901 }
const A: Array9F = ( 12.9172,  2.53718,  6.70003,  0.205855, 6.06791, 5.47913, 0.859041,11.6030, 1.45572 );
**)
{ Ge: maximum error 0.024. 0: 31.9774 }
const A: Array9F = ( 16.0816,  2.85090,  6.37470,  0.251600, 3.70680, 11.4468, 3.68300, 54.7625, 2.13130  );
{B:Array10F=(-0.462, 1.786, -0.717, 1.331, -1.163, 0.886, 0.081, 1.801, 0.228, 1.190);}
 B:Array10F=(-0.3858,1.7841,-0.6412,1.3291,-1.0885,0.8855,0.1547,1.8001,0.3016,1.1903);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFAl ( STL: float; Lambda: float; var ASF: Complex );
{ vzal jsem hodnotu pro Al. Je zde i hodnota Al/+3 }
{ Al: maximum error 0.018. 0: 12.9937 }
const A: Array9F = ( 6.42020,  3.03870,  1.90020,  0.74260,  1.59360, 31.5472, 1.96460, 85.0886, 1.11510 );
{B:Array10F=(0.318, 0.522, 0.269, 0.381, 0.204, 0.246, 0.056, 0.052, 0.032, 0.031);}
 B:Array10F=(0.3260,0.5212,0.2774,0.3807,0.2130,0.2455,0.0645,0.0514,0.0313,0.0522);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFGa (  STL: float; Lambda: float; var ASF: Complex );
{ vzal jsem hodnotu pro Ga. Je zde i hodnota Ga/+3 }
{ Ga: maximum error 0.025. 0: 30.9763 }
const A: Array9F = (15.2354,  3.06690,  6.70060,  0.241200, 4.35910, 10.7805, 2.96230, 61.4135, 1.71890 );
{B:Array10F=(-0.570,1.569, -0.841, 1.168, -1.354, 0.777, 0.163, 1.609, 0.249,1.059);}
 B:Array10F=(-0.4989,1.5674,-0.7701,1.1663,-1.2846,0.7763,0.2307,1.6083,0.3179,1.0589);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFAs ( STL: float; Lambda: float; var ASF: Complex );
{ pouze hodnota pro As: maximum error 0.025. 0: 32.9826 }
const A: Array9F = ( 16.6723,  2.63450,  6.07010,  0.264700, 3.43130, 12.9479, 4.27790, 47.7972, 2.53100 );
{B:Array10F=(-0.365, 2.022, -0.607, 1.508, -1.011, 1.006, -0.030, 2.007, 0.196, 1.332);}
 B:Array10F=(-0.2871,2.0194,-0.5260,1.5069,-0.9300,1.0051,-0.0499,2.0058,0.2758,1.3314);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFO ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F =(3.048500,13.277100,2.286800,5.701100,1.54630,0.323900,0.867000,32.908900,0.250800);
B:Array10F=(0.0928,0.0731,0.0716,0.0518,0.0492,0.0322,0.0106,0.0060,0.0056,0.0036);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFFe ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = (11.769500,4.761100,7.357300,0.307200,3.52220,15.353500,2.304500,76.880500,1.036900);
B:array10F=(-1.2935,0.7620,-2.0554,0.5649,-1.1336,3.1974,0.3463,0.8444,0.5448,0.3050);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFY ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = (17.776000,1.402900,10.294600,12.800600,5.72629,0.125599,3.265880,104.354000,1.912130);
B:array10F=(0.1263,3.9964,0.0138,3.0054,-0.2670,2.0244,-2.7962,3.5667,-0.2879,2.4099);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFNb ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = ( 17.614200, 1.188650, 12.014400, 11.766000, 4.04183, 0.204785, 3.533460, 69.795700, 3.755910 );
B:array10F=(0.1211,4.8761,0.1072,3.6768,-0.1121,2.4826,-2.0727,0.6215,-0.8282,2.8404);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFGd ( STL: float; Lambda: float; var ASF: Complex );
{ pouze hodnota pro As: maximum error 0.025. 0: 32.9826 }
const A: Array9F = (25.070900,2.253410,19.079800,0.181951,13.85180,12.933100,3.545450,101.398000,2.419600);
B:array10F=(-5.3778,5.7756,-7.1681,4.4280,-8.8380,11.9157,-0.1653,3.9035,-0.3741,2.5954);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFTb ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = (25.897600,2.242560,18.218500,0.196143,14.31670,12.664800,2.953540,115.362000,3.583420);
B:array10F=(-5.0951,6.1667,-6.5583,4.7292,-9.1472,9.1891,-0.1723,4.1537,-0.3496,2.7654);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;

procedure ASFW ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = (29.081800,1.720290,15.430000,9.225900,14.43270,0.321703,5.119820,57.056000,9.887500);
B:array10F=(-3.9270,10.3696,-4.2486,8.0005,-5.4734,5.5774,-0.8490,6.8722,-0.4201,4.6430);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;


procedure ASFSm ( STL: float; Lambda: float; var ASF: Complex );
const A: Array9F = (24.004200,2.472740,19.425800,0.196451,13.43960,14.39960,2.896040,128.00700,2.209630);
B:array10F=(-6.057,5.0744,-9.3497,3.8839,-5.3236,12.2178,-0.1638,3.4418,-0.4287,2.2815);
begin ASF1974(STL,A,ASF);
AddDispCorr(B,Lambda,ASF);
end;


function RtgZnaLatku ( Latka: String10 ): boolean;
var I: integer;
begin
Latka:=UpString(Latka); I:=0;
repeat inc(I);
until (I=KolikZnaRtg) or (Latka=UpString(CoZnaRtg[I]));
RtgZnaLatku:=Latka=UpString(CoZnaRtg[I]);
end;


function StringToLambda ( Lambda: string ): real;
var I: integer;
begin
Lambda:=UpString(Lambda);
for I:=1 to HowManyLambda do
  if Lambda=WhichLambdaStr[I] then
    begin StringToLambda:=WhichLambda[I]; exit end;
Beepni(2); StringToLambda:=CuKalfa
end;


end. { MFCHT }
